@@ -25,13 +25,16 @@ static network_callback callback_packetreceived;
 #define MAXRECONNECTS	10
 
 /* As above, except the server doesn't seem to be around at all. */
-#define MAXRECONNECTS_AWOL	1
+#define MAXRECONNECTS_AWOL	3
 
 /* Time to wait between each attempt. */
 static int reconnect_wait[MAXRECONNECTS + 1] = {
     0, 0, 1, 2, 4, 8, 15, 30, 60, 90, 90
 };
 
+/* Global tarsnap option declared in tarsnap_opt.h. */
+int tarsnap_opt_retry_forever = 0;
+
 /* Time before which we shouldn't print a "connection lost" warning. */
 static struct timeval next_connlost_warning = { 0, 0};
 
@@ -241,6 +244,8 @@ reconnect(NETPACKET_CONNECTION * NPC)
 
 	/* Have we lost our connection / failed to connect too many times? */
 	NPC->ndrops += 1;
+	if (tarsnap_opt_retry_forever && NPC->ndrops > MAXRECONNECTS)
+		NPC->ndrops = MAXRECONNECTS;
 	if ((NPC->ndrops > MAXRECONNECTS) ||
 	    (NPC->serveralive == 0 && NPC->ndrops > MAXRECONNECTS_AWOL)) {
 		warn0("Too many network failures");
