@@ -24,34 +24,34 @@ struct netproto_connect_cookie {
 	NETPROTO_CONNECTION * NC;
 };
 
-static network_callback callback_connect;
-static int failed(struct netproto_connect_cookie *, int);
-
 static int
 callback_connect(void * cookie, int s)
 {
 	struct netproto_connect_cookie * C = cookie;
+	int rc;
 
 	/* The connect is no longer pending. */
 	C->connect_cookie = NULL;
 
 	/* Did the connection attempt fail? */
-	if (s == -1)
-		return (failed(C, NETWORK_STATUS_CONNERR));
-
-	/* Inform the netproto code that we have a socket. */
-	if (netproto_setfd(C->NC, s)) {
-		/* Drop the socket. */
-		close(s);
-
-		/* Report the failure and clean up. */
-		return (failed(C, NETWORK_STATUS_ERR));
+	if (s == -1) {
+		/*
+		 * Call the upstream callback.  Upon being informed that the
+		 * connect has failed, the upstream code is responsible for
+		 * calling netproto_close, which (since we haven't called
+		 * netproto_setfd yet) will call into callback_cancel and let
+		 * us clean up.
+		 */
+		return ((C->callback)(C->cookie, NETWORK_STATUS_CONNERR));
 	}
 
+	/* Inform the netproto code that we have a socket. */
+	if (netproto_setfd(C->NC, s))
+		goto err2;
+
 	/* Perform key exchange. */
-	if (netproto_keyexchange(C->NC, C->useragent,
-	    C->callback, C->cookie))
-		return (failed(C, NETWORK_STATUS_ERR));
+	if (netproto_keyexchange(C->NC, C->useragent, C->callback, C->cookie))
+		goto err1;
 
 	/* Free the cookie. */
 	sock_addr_freelist(C->sas);
@@ -60,35 +60,38 @@ callback_connect(void * cookie, int s)
 
 	/* Success! */
 	return (0);
+
+err2:
+	/* Drop the socket since we can't use it properly. */
+	close(s);
+err1:
+	/* Call the upstream callback. */
+	rc = (C->callback)(C->cookie, NETWORK_STATUS_ERR);
+
+	/*
+	 * We've called netproto_setfd, so callback_cancel won't happen; we
+	 * are responsible for cleaning up after ourselves.
+	 */
+	sock_addr_freelist(C->sas);
+	free(C->useragent);
+	free(C);
+
+	/* Return value from user callback. */
+	return (rc);
 }
 
 static int
 callback_cancel(void * cookie)
 {
 	struct netproto_connect_cookie * C = cookie;
-
-	/* We're being called because NC is being closed already. */
-	C->NC = NULL;
-
-	/* We were cancelled. */
-	return (failed(C, NETWORK_STATUS_CANCEL));
-}
-
-static int
-failed(struct netproto_connect_cookie * C, int status)
-{
 	int rc;
 
 	/* Cancel the connection attempt if still pending. */
 	if (C->connect_cookie != NULL)
 		network_connect_cancel(C->connect_cookie);
 
-	/* Call the upstream callback. */
-	rc = (C->callback)(C->cookie, status);
-
-	/* Release the network protocol layer cookie if necessary. */
-	if (C->NC != NULL)
-		netproto_close(C->NC);
+	/* We were cancelled. */
+	rc = (C->callback)(C->cookie, NETWORK_STATUS_CANCEL);
 
 	/* Free our cookie. */
 	sock_addr_freelist(C->sas);
@@ -132,7 +135,7 @@ getserveraddr(void)
 	}
 
 	/* Return a duplicate of the cached value. */
-	return (sock_addr_duplist(srv_addr));
+	return (srv_addr ? sock_addr_duplist(srv_addr) : NULL);
 }
 
 /**
@@ -157,7 +160,7 @@ netproto_connect(const char * useragent,
 	C->cookie = cookie;
 
 	/* Look up the server's IP address. */
-	if ((C->sas = getserveraddr())== NULL)
+	if ((C->sas = getserveraddr()) == NULL)
 		goto err2;
 
 	/* Try to connect to server, waiting up to 5 seconds per address. */
