@@ -1,3 +1,5 @@
+#include <time.h>
+
 #include "archive.h"
 
 /**
@@ -10,14 +12,15 @@ void * archive_read_open_multitape(struct archive *, uint64_t, const char *);
 
 /**
  * archive_write_open_multitape(a, machinenum, cachedir, tapename, argc,
- *     argv, printstats, dryrun):
+ *     argv, printstats, dryrun, creationtime):
  * Open the multitape tape ${tapename} for writing and associate it with the
  * archive $a$.  If ${printstats} is non-zero, print archive statistics when
- * the tape is closed.  If ${dryrun} is non-zero, perform a dry run.  Return
- * a cookie which can be passed to the multitape layer.
+ * the tape is closed.  If ${dryrun} is non-zero, perform a dry run.
+ * Record ${creationtime} as the creation time in the archive metadata.
+ * Return a cookie which can be passed to the multitape layer.
  */
 void * archive_write_open_multitape(struct archive *, uint64_t, const char *,
-    const char *, int argc, char ** argv, int, int);
+    const char *, int argc, char ** argv, int, int, time_t);
 
 /**
  * archive_write_multitape_setmode(a, cookie, mode):
