@@ -169,7 +169,7 @@ storage_read_init(uint64_t machinenum)
 	S->cache_lru = NULL;
 	S->cache_mru = NULL;
 	S->cachesz = 0;
-	S->cachemaxsz = (size_t)(-1);
+	S->cachemaxsz = SIZE_MAX;
 
 	/* Open netpacket connection. */
 	if ((S->NPC = netpacket_open(USERAGENT)) == NULL)
@@ -433,13 +433,11 @@ storage_read_file_callback(STORAGE_R * S, uint8_t * buf, size_t buflen,
 
 	/* Ask the netpacket layer to send a request and get a response. */
 	if (netpacket_op(S->NPC, callback_read_file_send, C))
-		goto err1;
+		goto err0;
 
 	/* Success! */
 	return (0);
 
-err1:
-	free(C);
 err0:
 	/* Failure! */
 	return (-1);
@@ -536,8 +534,10 @@ callback_read_file_response(void * cookie, NETPACKET_CONNECTION * NPC,
 		case 1:
 			/* File is corrupt. */
 			sc = 2;
-			if (C->size == (uint32_t)(-1))
+			if (C->size == (uint32_t)(-1)) {
 				free(C->buf);
+				C->buf = NULL;
+			}
 			break;
 		case -1:
 			if (C->size == (uint32_t)(-1))
@@ -549,7 +549,7 @@ callback_read_file_response(void * cookie, NETPACKET_CONNECTION * NPC,
 		classname[0] = C->class;
 		memcpy(&classname[1], C->name, 32);
 		if (((CF = rwhashtab_read(C->S->cache, classname)) != NULL) &&
-		    (CF->inqueue != 0) && (CF->buf == NULL)) {
+		    (CF->inqueue != 0) && (CF->buf == NULL) && (sc == 0)) {
 			/* Make a copy of this buffer if we can. */
 			if ((CF->buf = malloc(C->buflen)) != NULL) {
 				/* Copy in data and data length. */
